#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define CTC1            WGM12           // Meilleur nom pour le bit

#define WAVE_SIZE	256
#define TIMER_DIVIDER	1
#define TEMPO_ACCEL	2

int8_t sinus[WAVE_SIZE];
int position=0;
volatile int cycles=0;

// Un cycle de sinus
static void computeSinus(){
int X=0,Y=8180;
for(int i=0;i<WAVE_SIZE;i++){
  X += (Y*4)/163;
  Y -= (X*4)/163;
  sinus[i]=128+(X>>6);
  }
}

static void init_minuteur(int diviseur){
TCCR1A=0;               // Le mode choisi n'utilise pas ce registre
TCCR1B=(1<<CTC1);       // Réinitialisation du minuteur sur expiration
switch(diviseur){
  case    1: TCCR1B |= (1<<CS10); break;
  case    8: TCCR1B |= (1<<CS11); break;
  case   64: TCCR1B |= (1<<CS11 | 1<<CS10); break;
  case  256: TCCR1B |= (1<<CS12); break;
  case 1024: TCCR1B |= (1<<CS12 | 1<<CS10); break;
  }
// Un cycle prend 1/F_CPU secondes.
// Un pas de compteur prend diviseur/F_CPU secondes.
// L'interruption est appelée toutes les OCR1A*diviseur/F_CPU secondes.
// Par ailleurs OCR1A est sur 16 bits donc avec un maximum de 65535.
// L'interruption peut donc être appelée entre F_CPU/diviseur fois par
// seconde et F_CPU/(65535*diviseur) par seconde.
TCNT1=0;                // Compteur initialisé
TIMSK1=(1<<OCIE1A);     // Comparaison du compteur avec OCR1A
}

static void prog_minuteur(unsigned int valeur){
cycles=0;
OCR1A=valeur;
}

void init_DAC(void){
computeSinus();
// Une période de sinus va prendre WAVE_SIZE*DIVIDER/F_CPU secondes.
// La fréquence maximale est F_CPU/DIVIDER/WAVE_SIZE et la
// fréquence minimale est F_CPU/DIVIDER/WAVE_SIZE/65535.
// Pour une fréquence f vous devez utiliser la valeur :
// valeur=F_CPU/DIVIDER/WAVE_SIZE/f
init_minuteur(TIMER_DIVIDER);
DDRF |= 0xe0;		// DAC0 à DAC2
DDRD |= 0xf0;		// DAC4, DAC3, DAC5, et DAC6
DDRB |= 0x10;		// DAC7
sei();
}

static void write_DAC(unsigned char data){
PORTF=(PORTF & 0x1f)|(data<<5);
PORTD=(PORTD & 0x0f)|(data & 0x10)|((data & 0x08) << 2)|((data & 0x60) << 1);
PORTB=(PORTB & 0xef)|((data & 0x80) >> 3);
}

ISR(TIMER1_COMPA_vect){ // Procédure d'interruption
write_DAC(sinus[position++]);
if(position==WAVE_SIZE){
  position=0;
  cycles++;
  }
}

void jouer_note(int note,int temps){
prog_minuteur(F_CPU/TIMER_DIVIDER/WAVE_SIZE/note);
int wait=note/TEMPO_ACCEL;
if(temps>0) wait *= temps;
if(temps<0) wait /= -temps;
while(cycles<wait);
}
